import json
from .elements import PlotElements


class PlotSections(object):
    def __init__(self, accqsure, plot_id):
        self.accqsure = accqsure
        self.plot_id = plot_id

    async def get(self, id_, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section/{id_}", "GET", kwargs
        )
        return PlotSection(self.accqsure, self.plot_id, **resp)

    async def list(self, limit=50, start_key=None, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section",
            "GET",
            {"limit": limit, "start_key": start_key, **kwargs},
        )
        plot_sections = [
            PlotSection(self.accqsure, self.plot_id, **plot_section)
            for plot_section in resp.get("results")
        ]
        return plot_sections, resp.get("last_key")


class PlotSection:
    def __init__(self, accqsure, plot_id, **kwargs):
        self.accqsure = accqsure
        self.plot_id = plot_id
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._heading = self._entity.get("heading")
        self._number = self._entity.get("number")
        self._style = self._entity.get("style")
        self._order = self._entity.get("order")
        self.elements = PlotElements(self.accqsure, self.plot_id, self._id)

    @property
    def id(self) -> str:
        return self._id

    @property
    def heading(self) -> str:
        return self._heading

    @property
    def number(self) -> str:
        return self._number

    @property
    def style(self) -> str:
        return self._style

    @property
    def order(self) -> int:
        return self._order

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"PlotSection( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/section/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self



import json
import logging
from accqsure.exceptions import SpecificationError


class PlotMarkers(object):
    def __init__(
        self,
        accqsure,
        plot_id,
        plot_waypoint_id,
    ):
        self.accqsure = accqsure
        self.plot_id = plot_id
        self.waypoint_id = plot_waypoint_id

    async def get(self, id_, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{id_}",
            "GET",
            kwargs,
        )
        return PlotMarker(
            self.accqsure, self.plot_id, self.waypoint_id, **resp
        )

    async def list(self, limit=50, start_key=None, **kwargs):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker",
            "GET",
            {"limit": limit, "start_key": start_key, **kwargs},
        )
        plot_markers = [
            PlotMarker(
                self.accqsure, self.plot_id, self.waypoint_id, **plot_marker
            )
            for plot_marker in resp.get("results")
        ]
        return plot_markers, resp.get("last_key")

    async def create(
        self,
        name,
        contents,
        **kwargs,
    ):

        data = dict(
            name=name,
            contents=contents,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info("Creating Plot Marker %s", name)

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker",
            "POST",
            None,
            payload,
        )
        plot_marker = PlotMarker(
            self.accqsure, self.plot_id, self.waypoint_id, **resp
        )
        logging.info("Created Plot Marker %s with id %s", name, plot_marker.id)

        return plot_marker

    async def remove(self, id_, **kwargs):

        await self.accqsure._query(f"/plot/{id_}", "DELETE", {**kwargs})


class PlotMarker:
    def __init__(self, accqsure, plot_id, plot_waypoint_id, **kwargs):
        self.accqsure = accqsure
        self.plot_id = plot_id
        self.waypoint_id = plot_waypoint_id
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._name = self._entity.get("name")
        self._status = self._entity.get("status")
        self._content_id = self._entity.get("content_id")

    @property
    def id(self) -> str:
        return self._id

    @property
    def status(self) -> str:
        return self._status

    @property
    def name(self) -> str:
        return self._name

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"PlotMarker( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):

        await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self._id}",
            "DELETE",
        )

    async def rename(self, name):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def _set_asset(self, path, file_name, mime_type, contents):
        return await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self.id}/asset/{path}",
            "PUT",
            params={"file_name": file_name},
            data=contents,
            headers={"Content-Type": mime_type},
        )

    async def get_contents(self):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not ready for plot marker"
            )

        resp = await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self.id}/asset/manifest.json",
            "GET",
        )
        return resp

    async def get_content_item(self, name):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not ready for plot marker"
            )

        return await self.accqsure._query(
            f"/plot/{self.plot_id}/waypoint/{self.waypoint_id}/marker/{self.id}/asset/{name}",
            "GET",
        )

    async def _set_content_item(self, name, file_name, mime_type, contents):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not ready for plot marker"
            )
        return await self._set_asset(name, file_name, mime_type, contents)



import json
import logging

from accqsure.exceptions import SpecificationError
from accqsure.manifests import Manifest


class Documents(object):
    def __init__(self, accqsure):
        self.accqsure = accqsure

    async def get(self, id_, **kwargs):

        resp = await self.accqsure._query(f"/document/{id_}", "GET", kwargs)
        return Document(self.accqsure, **resp)

    async def list(self, document_type_id, **kwargs):
        resp = await self.accqsure._query(
            "/document",
            "GET",
            dict(document_type_id=document_type_id, **kwargs),
        )

        documents = [
            Document(self.accqsure, **document)
            for document in resp.get("results")
        ]
        return documents, resp.get("last_key")

    async def create(
        self,
        document_type_id,
        name,
        doc_id,
        contents,
        **kwargs,
    ):

        data = dict(
            name=name,
            document_type_id=document_type_id,
            doc_id=doc_id,
            contents=contents,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info("Creating Document %s", name)

        resp = await self.accqsure._query("/document", "POST", None, payload)
        document = Document(self.accqsure, **resp)
        logging.info("Created Document %s with id %s", name, document.id)

        return document

    # async def markdown_convert(self, title, type, base64_contents, **kwargs):
    #     resp = await self.accqsure._query(
    #         f"/document/convert",
    #         "POST",
    #         None,
    #         {
    #             **kwargs,
    #             **dict(
    #                 title=title, type=type, base64_contents=base64_contents
    #             ),
    #         },
    #     )
    #     result = await self.accqsure._poll_task(resp.get("task_id"))
    #     return result.get("contents")

    async def remove(self, id_, **kwargs):

        await self.accqsure._query(
            f"/document/{id_}", "DELETE", dict(**kwargs)
        )


class Document:
    def __init__(self, accqsure, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._document_type_id = self._entity.get("document_type_id")
        self._name = self._entity.get("name")
        self._doc_id = self._entity.get("doc_id")
        self._content_id = self._entity.get("content_id")

    @property
    def id(self) -> str:
        return self._id

    @property
    def document_type_id(self) -> str:
        return self._document_type_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def doc_id(self) -> str:
        return self._doc_id

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"Document( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):

        await self.accqsure._query(
            f"/document/{self._id}",
            "DELETE",
        )

    async def rename(self, name):

        resp = await self.accqsure._query(
            f"/document/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):

        resp = await self.accqsure._query(
            f"/document/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def get_contents(self):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not uploaded for document"
            )

        resp = await self.accqsure._query(
            f"/document/{self.id}/asset/{self._content_id}/manifest.json",
            "GET",
        )
        return resp

    async def get_content_item(self, name):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not uploaded for document"
            )

        return await self.accqsure._query(
            f"/document/{self.id}/asset/{self._content_id}/{name}",
            "GET",
        )

    async def _set_asset(self, path, file_name, mime_type, contents):
        return await self.accqsure._query(
            f"/document/{self.id}/asset/{path}",
            "PUT",
            params={"file_name": file_name},
            data=contents,
            headers={"Content-Type": mime_type},
        )

    async def _set_content_item(self, name, file_name, mime_type, contents):
        if not self._content_id:
            raise SpecificationError(
                "content_id", "Content not finalized for inspection"
            )

        return await self._set_asset(
            f"{self._content_id}/{name}", file_name, mime_type, contents
        )

    async def list_manifests(self):

        resp = await self.accqsure._query(
            f"/document/{self.id}/manifest",
            "GET",
        )
        manifests = [Manifest(self.accqsure, **manifest) for manifest in resp]
        return manifests



import pathlib
import re
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

with (HERE / "accqsure" / "__init__.py").open() as fp:
    try:
        version = re.findall(r'^__version__ = "([^"]+)"\r?$', fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError("Unable to determine version.")

README = (HERE / "README.md").read_text()

with (HERE / "requirements.txt").open() as fp:
    install_requires = fp.read().splitlines()


setup(
    name="accqsure",
    version=version,
    description="AccQsure client SDK and command line utilities",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/AccQsure/python-sdk",
    author="AccQsure, Inc.",
    author_email="support@accqsure.ai",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    python_requires=">=3.10",
    install_requires=install_requires,
    entry_points="""
        [console_scripts]
        accqsure=accqsure.cli:cli
    """,
)
